<?php

declare(strict_types=1);

namespace App\Validator;

use App\Hasher;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;
use Symfony\Component\Validator\Exception\UnexpectedValueException;

final class CustomStringNeverCollidesValidator extends ConstraintValidator
{
    public function __construct(private Hasher $hasher)
    {
    }

    /**
     * @inheritDoc
     */
    public function validate(mixed $value, Constraint $constraint): void
    {
        if (!$constraint instanceof CustomStringNeverCollides) {
            throw new UnexpectedTypeException($constraint, CustomStringNeverCollides::class);
        }

        if (null === $value || '' === $value) {
            return;
        }

        if (!is_string($value)) {
            throw new UnexpectedValueException($value, 'string');
        }

        /** Can be decoded to id, so avoid it */
        if ($this->hasher->isValid($value)) {
            $this->context->buildViolation($constraint->message)->addViolation();
        }
    }
}
